package org.linotte.frame.cahier.timbre.ui.theme.simple;

import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.Composite;
import java.awt.Graphics2D;
import java.awt.Shape;
import java.awt.geom.Rectangle2D;

import org.linotte.frame.cahier.timbre.entite.Acteur;
import org.linotte.frame.cahier.timbre.entite.Bouton;
import org.linotte.frame.cahier.timbre.entite.Commentaire;
import org.linotte.frame.cahier.timbre.entite.Condition;
import org.linotte.frame.cahier.timbre.entite.Fonction;
import org.linotte.frame.cahier.timbre.entite.Pour;
import org.linotte.frame.cahier.timbre.entite.TantQue;
import org.linotte.frame.cahier.timbre.entite.e.Parfum;
import org.linotte.frame.cahier.timbre.ui.Planche;
import org.linotte.frame.cahier.timbre.ui.i.BoutonUI;
import org.linotte.frame.cahier.timbre.ui.i.PlancheUI;

public class BoutonSimpleUI implements BoutonUI {

	public Bouton moi;

	public BoutonSimpleUI(Bouton bouton) {
		moi = bouton;
		moi.largeur = Abaque.LARGEUR_TIMBRE;
		moi.hauteur = Abaque.LARGEUR_TIMBRE / 4 + 3;
	}

	@Override
	public void pre_dessine(PlancheUI planche) {
	}

	@Override
	public void dessine(PlancheUI planche) {

		Graphics2D g2d = ((Planche) planche).graphics;

		Composite old = g2d.getComposite();
		if (moi.glisser)
			g2d.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0.5f));

		Color couleur = Palette.VERBE.darker();
		if (moi.original instanceof Acteur) {
			switch (((Acteur) moi.original).role) {
			case TEXTE:
				couleur = Palette.ACTEUR_TEXTE.darker();
				break;
			case NOMBRE:
				couleur = Palette.ACTEUR_NOMBRE.darker();
				break;
			case FORMULE:
				couleur = Palette.ACTEUR_FORMULE.darker();
				break;
			}
		} else if (moi.original instanceof Condition) {
			couleur = Palette.CONDITION.darker();
		} else if (moi.original instanceof TantQue) {
			couleur = Palette.CONDITION.darker();
		} else if (moi.original instanceof Pour) {
			couleur = Palette.CONDITION.darker();
		} else if (moi.original instanceof Commentaire) {
			couleur = Palette.COMMENTAIRE.darker();
		} else if (moi.original instanceof Fonction) {
			couleur = Palette.FONCTION.darker();
		} else if (moi.parfum == Parfum.UNDO) {
			couleur = ((Planche) planche).historique.estceVide() ? Palette.UNDO_INACTIF : Palette.UNDO;
			int nb = ((Planche) planche).historique.taille();
			moi.label = "Annuler (" + nb + ")";
		}

		if (moi.survol && moi.zoneActive == null)
			g2d.setColor(couleur.brighter());
		else
			g2d.setColor(couleur);

		g2d.fillRoundRect(moi.x, moi.y, moi.largeur, moi.hauteur, Abaque.ANGLE_TIMBRE, Abaque.ANGLE_TIMBRE);

		g2d.setColor(Palette.TEXTE);
		g2d.drawString(moi.label, moi.x + 10, moi.y + 15);

		g2d.setComposite(old);

	}

	@Override
	public boolean clique(int x, int y) {
		return false;
	}

	@Override
	public boolean contient(int x, int y) {
		Shape shape = new Rectangle2D.Double(moi.x, moi.y, moi.largeur, moi.hauteur);
		return shape.contains(x, y);
	}

	@Override
	public void positionnerProchainTimbre() {
	}
}
